﻿//Box2D_09_02.as
package {
	//▼インポート
	import flash.display.Sprite;
	import Box2D.Collision.b2AABB;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Collision.Shapes.b2PolygonDef;
	import Box2D.Collision.Shapes.b2CircleDef
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2DebugDraw;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.display.SimpleButton;
	import flash.display.Bitmap;
	import flash.display.MovieClip;

	public class Box2D_09_02 extends Sprite {
		//▼定数定義
		//1mを表すピクセル数
		private static const DRAW_SCALE:Number = 100;
		//▼プロパティ定義
		private var _world:b2World;
		private var _exWall:b2Body;
		private var _bBall:b2Body;
		private var _bBox:b2Body;
		private var stepRotate:Number = Math.PI / 180 * 2;
		//▼コンストラクタ定義
		public function Box2D_09_02() {
			//▼物理エンジンのセットアップ
			//物理エンジン適用範囲の設定（ステージの100ピクセル外側）
			var worldRegion:b2AABB = new b2AABB();//矩形作成
			worldRegion.lowerBound.Set(-1, -100);
			worldRegion.upperBound.Set(6.5, 5);
			//重力の設定
			var gravity:b2Vec2 = new b2Vec2(0, 9.81);
			//var gravity:b2Vec2 = new b2Vec2(0, 0);
			//範囲・重力・スリープの有無を指定して物理ワールドを作成
			_world = new b2World(worldRegion, gravity, true);
			
			//▼壁（静体）の生成
			var wallW:Number = 5.5 / 2;
			var wallH:Number = 4 / 2;
			var wallT:Number = 0.02;
			var wall1:b2Body = xGetBox(_world, wallW, wallT, wallW, wallT);
			var wall2:b2Body = xGetBox(_world, wallT, wallH, (wallW - wallT) * 2, wallH);
			var wall3:b2Body = xGetBox(_world, wallW, wallT, wallW, (wallH - wallT) * 2);
			var wall4:b2Body = xGetBox(_world, wallT, wallH, wallT, wallH);
			//▼追加壁（中央で回転させる静体）の作成
			_exWall = xGetBox(_world, 1.5, 0.1, wallW, wallH, Math.PI / -6);

			//▼箱（動体）の生成
			_bBox = xGetBox(_world, 0.3, 0.2, 3.5, 0.4, 0, 0.5);
			_bBox.SetMassFromShapes();

			//▼ボール（動体）の生成
			var bdBall:b2BodyDef = new b2BodyDef();
			bdBall.position.Set(1.5, 1);
			bdBall.linearDamping = 0.5;
			bdBall.angularDamping = 0.5;
			//ShapeDefの設定
			var pdBall:b2CircleDef = new b2CircleDef();
			pdBall.radius = 0.3;
			pdBall.density = 1;//密度（1平方メートルあたりの重量（kg））
			pdBall.restitution = 0.5;//反発係数（通常0～1）
			pdBall.friction = 0.5;//摩擦係数
			//ボディの生成
			_bBall = _world.CreateBody(bdBall);
			_bBall.CreateShape(pdBall);
			//質量の設定
			_bBall.SetMassFromShapes();
			
			//▼表示用オブジェクトの生成と設定
			//ステージ上に配置したムービークリップインスタンス（exWall_mc）を追加壁に関連付ける
			_exWall.SetUserData(exWall_mc);
			
			//埋め込みアセットクラス化したムービークリップシンボルを箱に関連付ける
			sp = new McRect();//埋め込みアセットクラス化したムービークリップシンボルのインスタンスを生成
			this.addChild(sp);//メインタイムラインに追加
			_bBox.SetUserData(sp);//箱に関連付ける
			
			//ビットマップをボールに関連付ける
			var sp:Sprite = new Sprite();//Bitmapオブジェクトを配置するためのSpriteを生成
			var bm:Bitmap = new Bitmap(new BmpCircle(0, 0));//ライブラリのビットマップをBitmapインスタンスに関連付ける
			//Bitmapオブジェクトの配置位置を画像の中心になるよう変更
			bm.x = -(bm.width / 2);
			bm.y = -(bm.height / 2);
			sp.addChild(bm);//BitmapオブジェクトをSpriteオブジェクトに追加
			this.addChild(sp);//Spriteオブジェクトをメインタイムラインに追加
			_bBall.SetUserData(sp);//ビットマップを含むSpriteオブジェクトをボールに関連付ける
			
			//▼描画設定
			var dd:b2DebugDraw = new b2DebugDraw();
			dd.m_sprite = this;
			dd.m_drawScale = DRAW_SCALE; //1mを100ピクセルにする
			dd.m_fillAlpha = 0.3; //塗りのアルファ値
			dd.SetFlags(b2DebugDraw.e_shapeBit);//物体を表示する設定に
			_world.SetDebugDraw(dd);//物理ワールドにセット
			
			//▼イベントリスナーの追加
			addEventListener(Event.ENTER_FRAME, _enterFrameHandler);
			hit_btn.addEventListener(MouseEvent.CLICK, _hit_btn_clickHandler);
		}
		
		//▼メソッド定義
		//Boxの生成
		private function xGetBox(world_:b2World, w_:Number, h_:Number, x_:Number, y_:Number, rot_:Number = 0, d_:Number = 0) {
			//BodyDefの設定
			var bodyDef:b2BodyDef = new b2BodyDef();
			bodyDef.position.Set(x_, y_);
			bodyDef.angle = rot_;
			//ShapeDefの設定
			var polyDef:b2PolygonDef = new b2PolygonDef();
			polyDef.density = d_;
			polyDef.SetAsBox(w_, h_);
			//ボディの生成
			var box:b2Body = world_.CreateBody(bodyDef);
			box.CreateShape(polyDef);
			return box;
		}
		
		//▼イベントリスナー定義
		private function _enterFrameHandler(eventObj:Event):void {
			//追加壁の回転
			var exAngle:Number = _exWall.GetAngle();
			var ecPosition:b2Vec2 = _exWall.GetPosition();
			_exWall.SetXForm(ecPosition, exAngle + stepRotate);
			
			//物理シミュレーションの更新
			_world.Step(1 / 24, 10);
			
			//Spriteオブジェクトの表示更新
			for (var b:b2Body = _world.GetBodyList(); b; b = b.GetNext()) {
				if (b.GetUserData() is Sprite) {
					//物理エンジン内での位置と回転角度を反映させる
					b.GetUserData().x = b.GetPosition().x * DRAW_SCALE;
					b.GetUserData().y = b.GetPosition().y * DRAW_SCALE;
					b.GetUserData().rotation = b.GetAngle() * 180 / Math.PI;
				}
			}
		}
		//以下ボタン用イベントリスナー
		private function _hit_btn_clickHandler(eventObj:Event):void {
			var ballC:b2Vec2 = _bBall.GetWorldCenter().Copy();
			var ballVecX:Number = Math.random() * 4 - 2;
			var ballVecY:Number = Math.random() * 3 + 1;
			_bBall.ApplyImpulse(new b2Vec2(ballVecX, -ballVecY), ballC);
			
			var boxC:b2Vec2 = _bBox.GetWorldCenter().Copy();
			var boxVecX:Number = Math.random() * 4 - 2;
			var boxVecY:Number = Math.random() * 3 + 1;
			_bBox.ApplyImpulse(new b2Vec2(boxVecX, -boxVecY), boxC);
		}
	}
}